/*****************************************************************************
*																			 *
*		    Network Interface Messages										 *
*																			 *
*  file: ni_msg.h      date: 27/10/92									gw   *
*****************************************************************************/

#ifndef NI_MSG_H
#define NI_MSG_H

#define NEURON_ID_LEN 6

#define LFG_MIP_TAG 1
#define CBI_UPDATE  2
#define HLI_UPDATE  3

typedef unsigned char msg_tag;

/****************************************************************************
 * Network Interface Command data structure.  This is the application-layer
 * header used for all messages to and from a LONWORKS network interface.
 ****************************************************************************/

/* Literals for the 'cmd.q.queue' nibble of NI_Hdr.							*/
typedef enum {
    niTQ		=  2, 			/* Transaction queue						*/
    niTQ_P		=  3, 			/* Priority transaction queue				*/
    niNTQ		=  4, 			/* Non-transaction queue					*/
    niNTQ_P		=  5, 			/* Priority non-transaction queue			*/
    niRESPONSE	=  6, 			/* Response msg & completion event queue	*/
    niINCOMING	=  8		    /* Received message queue					*/
} NI_Queue;

/* Literals for the 'cmd.q.q_cmd' nibble of NI_Hdr. */
typedef enum {
    niCOMM	     = 1,			/* Data transfer to/from network			*/
    niNETMGMT	 = 2			/* Local network management/diagnostics		*/
} NI_QueueCmd;

/* Literals for the 'cmd.noq' byte of NI_Hdr. */
typedef enum {
    niNULL			 = 0x00,
    niRESET			 = 0x50,
    niFLUSH_COMPLETE = 0x60,	/* Uplink									*/
    niFLUSH_CANCEL   = 0x60,	/* Downlink 								*/
    niONLINE	     = 0x70,
    niOFFLINE	     = 0x80,
    niFLUSH			 = 0x90,
    niFLUSH_IGN      = 0xA0,
    niACK			 = 0xC0,
    niPUPXOFF	     = 0xE1,
    niPUPXON	     = 0xE2
} NI_NoQueueCmd;

/* Header for network interface messages.  The header is a union of
 * two command formats: the 'q' format is used for the niCOMM and
 * niNETMGMT commands that require a queue specification; the 'noq'
 * format is used for all other network interface commands.
 * Both formats have a length specification where:
 *
 *	length = header (3) + address field (11 if present) + data field
 *
 */
typedef union {
    struct {
		unsigned char queue	:4;		/* Network interface message queue	    */
									/* Use value of type 'NI_Queue'         */
		unsigned char q_cmd	:4;		/* Network interface command with queue */
									/* Use value of type 'NI_QueueCmd'      */
		unsigned char length;	    /* Length of the buffer to follow	    */
    } q;							/* Queue option							*/
    struct {
		unsigned char cmd;			/* Network interface command w/o queue  */
									/* Use value of type 'NI_NoQueueCmd'    */
		unsigned char length;	    /* Length of the buffer to follow	    */
    } noq;							/* No queue option						*/
} NI_Hdr;

/*
 ****************************************************************************
 * Message Header structure for sending and receiving explicit
 * messages and network variables which are not processed by the
 * network interface (host selection enabled).
 ****************************************************************************
 */

/* Literals for 'st' fields of ExpMsgHdr and NetVarHdr.						*/

typedef enum {
   ACKD 			= 0,
   UNACKD_RPT		= 1,
   UNACKD			= 2,
   REQUEST			= 3
} ServiceType;

/* Literals for 'cmpl_code' fields of ExpMsgHdr and NetVarHdr.				*/

typedef enum {
    MSG_NOT_COMPL	= 0, 			/* Not a completion event				*/
    MSG_SUCCEEDS	= 1, 			/* Successful completion event			*/
    MSG_FAILS		= 2				/* Failed completion event				*/
} ComplType;

/* Explicit message and Unprocessed NV Application Buffer.					*/

typedef struct {

    unsigned char tag	    :4;     /* Message tag for implicit addressing  */
									/* Magic cookie for explicit addressing */
    unsigned char auth	    :1;     /* 1 => Authenticated					*/
    unsigned char st	    :2;     /* Service Type - see 'ServiceType'     */
    unsigned char msg_type  :1;     /* 0 => explicit message				*/
									/*	    or unprocessed NV				*/
/*--------------------------------------------------------------------------*/
    unsigned char response  :1;     /* 1 => Response, 0 => Other			*/
    unsigned char pool	    :1;     /* 0 => Outgoing						*/
    unsigned char alt_path  :1;     /* 1 => Use path specified in 'path'    */
									/* 0 => Use default path				*/
    unsigned char addr_mode :1;     /* 1 => Explicit addressing,			*/
									/* 0 => Implicit						*/
									/* Outgoing buffers only				*/
    unsigned char cmpl_code :2;     /* Completion Code - see 'ComplType'    */
    unsigned char path	    :1;     /* 1 => Use alternate path, 			*/
									/* 0 => Use primary path				*/
									/*	    (if 'alt_path' is set)          */
    unsigned char sPriority  :1;	/* 1 => Priority message				*/
/*--------------------------------------------------------------------------*/
    unsigned char length;			/* Length of msg or NV to follow	    */
									/* not including any explicit address   */
									/* field, includes code byte or			*/
									/* selector bytes						*/
} ExpMsgHdr;

/*
 ****************************************************************************
 * Message Header structure for sending and receiving network variables
 * that are processed by the network interface (network interface
 * selection enabled).
 ****************************************************************************
 */

typedef struct {
    unsigned char tag	    :4;     /* Magic cookie for correlating			*/
									/* responses and completion events	    */
    unsigned char rsvd0     :2;
    unsigned char poll	    :1;     /* 1 => Poll, 0 => Other				*/
    unsigned char msg_type  :1;     /* 1 => Processed network variable	    */
/*--------------------------------------------------------------------------*/
    unsigned char response  :1;     /* 1 => Poll response, 0 => Other	    */
    unsigned char pool	    :1;     /* 0 => Outgoing						*/
    unsigned char trnarnd   :1;     /* 1 => Turnaround Poll, 0 => Other     */
    unsigned char addr_mode :1;     /* 1 => Explicit addressing,			*/
									/* 0 => Implicit addressing 			*/
    unsigned char cmpl_code :2;     /* Completion Code - see above			*/
    unsigned char path	    :1;     /* 1 => Used alternate path 			*/
									/* 0 => Used primary path				*/
									/*	    (incoming only)					*/
    unsigned char sPriority  :1;	/* 1 => Priority msg (incoming only)    */
/*--------------------------------------------------------------------------*/
    unsigned char length;			/* Length of network variable to follow */
									/* not including any explicit address   */
									/* not including index and rsvd0 byte   */
} NetVarHdr;

/* Union of all message headers. */

typedef union {
    ExpMsgHdr  exp;
    NetVarHdr  pnv;
} MsgHdr;

/*
 ****************************************************************************
 * Network Address structures for sending messages with explicit addressing
 * enabled.
 ****************************************************************************
 */

/* Literals for 'type' field of destination addresses for outgoing messages. */

typedef enum {
    UNASSIGNED	   = 0,
    SUBNET_NODE    = 1,
    NEURON_ID	   = 2,
    BROADCAST	   = 3,
    IMPLICIT       = 126,    /* not a real destination type */
    LOCAL          = 127     /* not a real destination type */
} AddrType;

/* Group address structure.  Use for multicast destination addresses.	*/

typedef struct {
    unsigned char size	    :7; 	/* Group size (0 => huge group)     */
    unsigned char type	    :1; 	/* 1 => Group						*/

    unsigned char member    :6; 	/* Member ID (0 => huge group)	    */
    unsigned char rsvd0     :1;
    unsigned char domain    :1; 	/* Domain index 					*/

    unsigned char retry     :4; 	/* Retry count						*/
    unsigned char rpt_timer :4; 	/* Retry repeat timer				*/

    unsigned char tx_timer  :4; 	/* Transmit timer index 			*/
    unsigned char rsvd1     :4;

    unsigned char sGroup;			/* Group ID							*/
} SendGroup;

/* Subnet/node ID address.  Use for a unicast destination address.		*/

typedef struct {
    unsigned char type; 			/* SUBNET_NODE						*/

    unsigned char node	    :7; 	/* Node number						*/
    unsigned char domain    :1; 	/* Domain index 					*/

    unsigned char retry     :4; 	/* Retry count						*/
    unsigned char rpt_timer :4; 	/* Retry repeat timer				*/

    unsigned char tx_timer  :4; 	/* Transmit timer index 			*/
    unsigned char rsvd1     :4;

    unsigned char subnet;			/* Subnet ID						*/
} SendSnode;

/* 48-bit NEURON ID destination address.								*/

typedef struct {
    unsigned char type; 			/* NEURON_ID						*/

    unsigned char rsvd0     :7;
    unsigned char domain    :1; 	/* Domain index 		    */

    unsigned char retry     :4; 	/* Retry count			    */
    unsigned char rpt_timer :4; 	/* Retry repeat timer		    */

    unsigned char tx_timer  :4; 	/* Transmit timer index 	    */
    unsigned char rsvd2     :4;

    unsigned char subnet;		/* Subnet ID, 0 => pass all routers	*/
    unsigned char nid[NEURON_ID_LEN];	/* NEURON ID			    */
} SendNrnid;

/* Broadcast destination address. */

typedef struct {
    unsigned char type; 	    /* BROADCAST			    */

    unsigned char backlog   :6;     /* Backlog				    */
    unsigned char rsvd0     :1;
    unsigned char domain    :1;     /* Domain index			    */

    unsigned char retry     :4;     /* Retry count			    */
    unsigned char rpt_timer :4;     /* Retry repeat timer		    */

    unsigned char tx_timer  :4;     /* Transmit timer index		    */
    unsigned char rsvd2     :4;

    unsigned char subnet;	    /* Subnet ID, 0 => domain-wide	    */
} SendBcast;

/* Address format to clear an address table entry.	   */
/* Sets the first 2 bytes of the address table entry to 0. */

typedef struct {
    unsigned char type; 	   /* UNASSIGNED			   */
} SendUnassigned;

/* Union of all destination addresses. */

typedef union {
    SendGroup	   gp;
    SendSnode	   sn;
    SendBcast	   bc;
    SendNrnid	   id;
    SendUnassigned ua;
} SendAddrDtl;

/*
 ****************************************************************************
 * Network Address structures for receiving messages with explicit
 * addressing enabled.
 ****************************************************************************
 */

/* Received subnet/node ID destination address.  Used for unicast messages. */

typedef struct {
    unsigned char subnet;
    unsigned char node	    :7;
    unsigned char			:1;
} RcvSnode;

/* Received 48-bit NEURON ID destination address. */

typedef struct {
    unsigned char subnet;
    unsigned char nid[ NEURON_ID_LEN ];
} RcvNrnid;

/* Union of all received destination addresses. */

typedef union {
    unsigned char gp;			/* Group ID for multicast destination		*/
    RcvSnode	  sn;			/* Subnet/node ID for unicast				*/
    RcvNrnid	  id;			/* 48-bit NEURON ID destination address		*/
    unsigned char subnet;		/* Subnet ID for broadcast destination		*/
								/* 0 => domain-wide							*/
} RcvDestAddr;

/* Source address of received message.	Identifies */
/* network address of node sending the message.    */

typedef struct {
    unsigned char subnet;
    unsigned char node	    : 7;
    unsigned char			: 1;
} RcvSrcAddr;

/* Literals for the 'format' field of RcvAddrDtl. */

typedef enum {
    ADDR_RCV_BCAST  = 0,
    ADDR_RCV_GROUP  = 1,
    ADDR_RCV_SNODE  = 2,
    ADDR_RCV_NRNID  = 3
} RcvDstAddrFormat;

/* Address field of incoming message. */

typedef struct {
    unsigned char format      :6;   /* Destination address type 			*/
									/* See 'RcvDstAddrFormat'               */
    unsigned char flex_domain :1;   /* 1 => broadcast to unconfigured node  */
    unsigned char domain      :1;   /* Domain table index					*/

    RcvSrcAddr	source; 			/* Source address of incoming message   */
    RcvDestAddr dest;				/* Destination address of incoming msg  */
} RcvAddrDtl;

/*
 ****************************************************************************
 * Network Address structures for receiving responses with explicit
 * addressing enabled.
 ****************************************************************************
 */

/* Source address of response message. */

typedef struct {
    unsigned char subnet;
    unsigned char node	   : 7;
    unsigned char is_snode : 1;     /* 0 => Group response,		    */
									/* 1 => snode response		    */
} RespSrcAddr;

/* Destination of response to unicast request. */

typedef struct {
    unsigned char subnet;
    unsigned char node		: 7;
    unsigned char			: 1;
} RespSnode;

/* Destination of response to multicast request. */

typedef struct {
    unsigned char subnet;
    unsigned char node		: 7;
    unsigned char			: 1;
    unsigned char sGroup;
    unsigned char member	: 6;
    unsigned char			: 2;
} RespGroup;

/* Union of all response destination addresses. */

typedef union {
    RespSnode  sn;
    RespGroup  gp;
} RespDestAddr;

/* Address field of incoming response. */

typedef struct {
    unsigned char				:6;
    unsigned char flex_domain	:1;   /* 1=> Broadcast to unconfigured node   */
    unsigned char domain		:1;   /* Domain table index		    */

    RespSrcAddr  source;				/* Source address of incoming response  */
    RespDestAddr dest;					/* Destination address of incoming resp */
} RespAddrDtl;

/* Explicit address field if explicit addressing is enabled. */

typedef union {
    RcvAddrDtl	rcv;
    SendAddrDtl snd;
    RespAddrDtl rsp;
} ExplicitAddr;

/*
 ****************************************************************************
 * Data field structures for explicit messages and network variables.
 ****************************************************************************
 */

/*
 * MAX_NETMSG_DATA specifies the maximum size of the data portion of an
 * application buffer.	MAX_NETVAR_DATA specifies the maximum size of the
 * data portion of a network variable update.  The values specified here
 * are the absolute maximums,based on the LONTALK protocol. Actual limits
 * are based on the buffer sizes defined on the attached NEURON CHIP.
 */

#define MAX_NETMSG_DATA 229
#define MAX_NETVAR_DATA 31

// Data field for network variables (host selection enabled)
typedef struct {
    unsigned char NV_selector_hi	:6;
    unsigned char direction			:1;	    // 1 => outp NV, 0 => inp NV 
    unsigned char must_be_one		:1;	    // Must be set to 1 
    unsigned char NV_selector_lo;
    unsigned char _data[MAX_NETVAR_DATA];   // Network variable data
} UnprocessedNV;

// Data field for network variables (network interface selection enabled)
typedef struct {
    unsigned char index;					// Index into NV config table
    unsigned char rsvd0;
    unsigned char _data[MAX_NETVAR_DATA];   // Network variable data
} ProcessedNV;

// Data field for explicit messages
typedef struct {
    unsigned char _code;					// Message code
    unsigned char _data[MAX_NETMSG_DATA];   // Message data
} ExplicitMsg;

// Union of all data fields
typedef union {
    UnprocessedNV unv;
    ProcessedNV   pnv;
    ExplicitMsg   exp;
} MsgData;

/*
typedef union {
    struct {
        UCHAR    _code   ;
        UCHAR    data_0 ;
    } exp ;
    struct {
        UCHAR    index  ;
        UCHAR    rsvd0  ;
    } pnv ;
  
    struct {
        UCHAR   NV_selector_hi : 6 ;
        UCHAR   direction      : 1 ;
     	UCHAR   must_be_one    : 1 ;
        UCHAR   NV_selector_lo ;
    } unv ;
} HdrData ;
*/

/*
// Application buffer when using explicit addressing

typedef struct {
    NI_Hdr	 ni_hdr;				// Network interface header
    MsgHdr	 msg_hdr;				// Message header
    ExplicitAddr addr;				// Network address
    MsgData	 _data; 				// Message data
} ExpAppBuffer;
*/
//----------------------------------------------------------------------------

#endif
